# Logan McIntyre, STEMBoT Inc, 20191210
# A simple reaction timing game using an LED Plug-n-Play module
# The game starts by flashing a random LED three times. The LEDs
# will then flash in sequence until the player presses the S4 switch.
# If the player missed the correct LED, all of them will briefly flash.
# If the player gets the right LED, all of the LEDs will flash three times.
# The game can then be reset by pressing the S4 switch again. 

import graphics
import color
import pnp
import machine
from random import randint
import uasyncio

flashSpeed = 100		# how much time (in milliseconds) each led blinks

port=pnp.GPIO("top")
button=machine.Pin('B5',machine.Pin.IN,machine.Pin.PULL_UP)

bgColor = color.BLUE
textColor = color.WHITE

def explanation():
	graphics.paint(bgColor)
	graphics.arrow(270,8,220,20,textColor)	#point to A button
	graphics.arrow(90,310,5,20,textColor)	#point to top PnP port
	graphics.print("Plug in LED module here",100,16,textColor)
	graphics.print("Press the A button to begin.",10,170,textColor)
	graphics.print("The winning LED will flash 3 times.",10,180,textColor)
	graphics.print("Press the A button again once",10,190,textColor)
	graphics.print("the winning LED lights up to win!",25,200,textColor)

async def main():		#standard header
	explanation()
	playing = False
	win = False
	while(1):
		while playing:
			port.allOff()
			win=False
			winningNumber=randint(0,7)	#get a random number between 0 and 7

			#flash the correct LED three times before starting
			n = 0
			while n < 3:
				port.pin[winningNumber].on()
				await uasyncio.sleep_ms(500)	#half second delay
				port.pin[winningNumber].off()
				await uasyncio.sleep_ms(500)
				n = n+1
	
			#blink each LED in sequence, starting with D1 (pin[0])
			ledNumber=0
			while not win:
				port.pin[ledNumber].on()
				time = 0
				while time < flashSpeed:
					await uasyncio.sleep_ms(1)
					if button.value() == 0:	
						if ledNumber == winningNumber:
							playing = False
							win = True
						else:
							port.allOn()
							await uasyncio.sleep_ms(250)
							port.allOff()
					time=time+1
				port.pin[ledNumber].off()
				ledNumber=ledNumber+1
				if ledNumber > 7:
					ledNumber = 0
		
		if win is True:
			cycle = 3
			while cycle > 0:
				port.allOn()
				await uasyncio.sleep_ms(500)
				port.allOff()
				await uasyncio.sleep_ms(500)
				cycle=cycle-1
	
		while not playing:
			if button.value() == 0:
				playing = True
				win = False
			await uasyncio.sleep_ms(1)