import sb
import remote
import uasyncio
import graphics
import color
import time
import switch

#=======USER DEFINED VALUES==========
maxSpeed = 500		#motor max speed (absolute max is 1000)
driveMode = -1		#-1 for front wheel drive, 1 for rear wheel drive
precisionDrivingMode = False		#set to True to prevent SB2 from rolling (drains battery faster)
#====================================

leftMotor=sb.Motor(2)
rightMotor=sb.Motor(1)
leftMotor.brake_mode(precisionDrivingMode)
rightMotor.brake_mode(precisionDrivingMode)
r=remote.Remote()
pinBack=switch.Switch('c')		#rightmost button below LCD
buttonUp=switch.Switch('up')
buttonDn=switch.Switch('down')
bgColor=color.RED
textColor=color.WHITE

speedFactor=int(driveMode*maxSpeed/128)


async def main():
	selected=""

	def motorsOff():
		leftMotor.sleep(True)
		rightMotor.sleep(True)

	def motorsOn():
		leftMotor.sleep(False)
		rightMotor.sleep(False)

	graphics.fill_rectangle(0,0,graphics.lcdwidth,graphics.lcdheight,bgColor)

	if not r.is_paired():
		graphics.print("Error: Remote not paired.",70,100,textColor)
		time.sleep_ms(3000)
	else:
		graphics.paint(bgColor)
		graphics.print("Press Up for two joystick driving mode",20,90,textColor)
		graphics.print("Press Down for one joystick driving mode",20,110,textColor)
		time.sleep_ms(400)
		while selected is "":
			if buttonUp.value():
				selected='A'
			elif buttonDn.value():
				selected='B'

	graphics.paint(bgColor)
	rdata=remote.RemoteData()
	#prevLeftSpeed=0
	#prevRightSpeed=0
	motorsOn()
	exitRC=False

	if selected is 'A':
		while not exitRC:
			try:
				r.read(rdata,50)
				if (rdata.buttons & remote.STRT_BIT) or pinBack.value():
					exitRC=True
					print("exiting")
				graphics.print("Running two joystick mode",20,130,textColor)
				graphics.fill_rectangle(35,150,50,30,bgColor)
				graphics.print("L: {}".format(rdata.ljoy_up_down),20,150,textColor)
				graphics.print("R: {}".format(rdata.rjoy_up_down),20,165,textColor)
				leftSpeed=int(speedFactor*rdata.ljoy_up_down)
				rightSpeed=int(-1*speedFactor*rdata.rjoy_up_down)
				#if (leftSpeed != prevLeftSpeed):
				leftMotor.speed(leftSpeed,1)
				#if (rightSpeed != prevRightSpeed):
				rightMotor.speed(rightSpeed,1)
				#prevLeftSpeed=leftSpeed
				#prevRightSpeed=rightSpeed
			except Exception as e:
				print("Exception: No radio data")
				if pinBack.value():
					exitRC=True
			await uasyncio.sleep_ms(5)
		prevRightSpeed = 0
		prevLeftSpeed = 0
		motorsOff()
	elif selected is 'B':
		while not exitRC:
			try:
				r.read( rdata, 50 )
				if (rdata.buttons & remote.STRT_BIT) or pinBack.value():
					exitRC=True
				y_joystick = rdata.ljoy_up_down * 100 / 255
				x_joystick = rdata.ljoy_left_right * 100 / 255
				dataUD = rdata.ljoy_up_down
				dataLR = rdata.ljoy_left_right

				graphics.fill_rectangle(35,150,50,30,bgColor)
				graphics.print("U/D: {}".format(dataUD),20,150,textColor)
				graphics.print("L/R: {}".format(dataLR),20,165,textColor)

				lspeed=driveMode*((3*dataUD)+(2*dataLR))
				rspeed=driveMode*((3*dataUD)-(2*dataLR))	
	
				leftMotor.speed(lspeed,1)
				rightMotor.speed(rspeed,1)
				await uasyncio.sleep_ms(1)
			except Exception as e:
				print("Exception: No radio data")
				if pinBack.value():
					exitRC = True
				await uasyncio.sleep_ms( 1 )
		motorsOff()