import sys
import machine
import sb
from pyb import LED
from machine import Pin
import graphics 
import color
import os
from rtttl import RTTTL
import buzzer
import time
import utime
import uasyncio as asyncio
import remote
from struct import *
from label import Label
from menu import Menu, MenuItem
from batteryicon import BatteryIcon
from keycallback import KeyCallback
import remotetest
#import globalCommander
import RC
import switch
from appcontainer import AppContainer

border_offset_y = 20	#just below status bar
statusBgColor = color.VAMPIRE_BLACK
statusTextColor = color.MALACHITE
bgColor = color.RED
textColor = color.WHITE
warnColor = color.ORANGE
exit=False		
valid=[]
appRunning=False	   

#store a pointer to the graphics instance in 
#the graphics module for apps to use without
#having to create it
#graphics.lcd=g 
#graphics.textColor=textColor

def _handle_exception_and_stop(loop, context):
	print(context["message"])
	print("future:", context["future"], "coro=", context["future"].coro)
	sys.print_exception(context["exception"])
	
	graphics.print("Failed to run! See REPL for details.", 10,120, color.WHITE)
	 
async def status():
	global appRunning
	while True:
		#ticks=utime.ticks_ms()			
		#sec='%.0f'%(ticks/1000)			
		#min=int(int(sec)/60)	
		#sec=int(sec)-(int(min)*60)
		
		graphics.setViewport(0,0,graphics.lcdwidth,graphics.lcdheight)#OS context
		#ltime.setText('T:'+str(min)+'m'+str(sec)+'s')
		#ltime.paint()
		lbatt.update()
		if(pinVBUS.value()==1):
			graphics.print("USB", 250,2, textColor)
		else:
			graphics.erase(3,250,0,statusBgColor)
		
		if(pinUEXTFLT2.value()==0):
			graphics.print("3", 220+16,2, bgColor)
		else:
			graphics.erase(1,220+16,0,statusBgColor)
		if(pinUEXTFLT1.value()==0):
			graphics.print("2", 220+8,2, bgColor)
		else:
			graphics.erase(1,220+8,0,statusBgColor)
		if(pinUEXTFLT0.value()==0):
			graphics.print("1", 220,2, bgColor)
		else:
			graphics.erase(1,220,0,statusBgColor)
		
		
		if(appRunning):
			graphics.setViewport(0,15,graphics.lcdwidth,graphics.lcdheight-15)#app context   
		else:
			graphics.setViewport(0,0,graphics.lcdwidth,graphics.lcdheight)#OS context
		await asyncio.sleep_ms(500)
	
async def keys():
	while(True):
		if(switch.Switch('b').value()):
			selectCallback.invoke()
		elif(switch.Switch('up').value()):
			upCallback.invoke()
		elif(switch.Switch('down').value()):
			downCallback.invoke()
		elif(switch.Switch('c').value()):
			backCallback.invoke()
		await asyncio.sleep_ms(200)
		

leds = [LED(i) for i in range(1,4)]

tune=RTTTL('14:d=4,o=6,b=180:c,e,g')
#buzzer.play(tune)	#disabled for testing

lsb=Label( 0, 0, 70,15, statusBgColor, statusTextColor, 'STEMBoT2')
#ltime=Label(80, 0, 70,15, statusBgColor, statusTextColor, '')
lpair=Label( 80, 0, 70,15, statusBgColor, statusTextColor, '')
lbatt=BatteryIcon( 275, 0, statusBgColor, statusTextColor)

graphics.fill_rectangle(0,0,graphics.lcdwidth,graphics.lcdheight,bgColor)
graphics.rectangle(8,8,graphics.lcdwidth-16,graphics.lcdheight-16, textColor)
graphics.print("Hello STEMBot 2!", 100, 100, textColor)
time.sleep_ms(250)	#intro screen delay


pinVBUS=Pin('A9',Pin.IN,Pin.PULL_DOWN)
pinUEXTFLT0=Pin('A4',Pin.IN,Pin.PULL_UP)
pinUEXTFLT1=Pin('G8',Pin.IN,Pin.PULL_UP)
pinUEXTFLT2=Pin('F1',Pin.IN,Pin.PULL_UP)

#Two ways to find apps to cover different use-cases. 
# Method 1: Just look for modules in /apps with a main function. Keeps app syntax lightweight. 
# Method 2: Similar to OS like android with a centeral manifest. 
#           Each app has private dir, and each app is a Python class (rather than just a function). 
#           This approach is more complex but may be more suitable for more sophisticated apps.

# == Method 2 ==
try:
	with open('apps/manifest.txt', 'r') as fp:
		Lines = fp.readlines() 
		for line in Lines: 
			mod = __import__(line.strip())
			print(line)
			print(str(mod))
			ac = AppContainer(mod.Main, True)
			ac.setAttributes(mod.name, mod.author,mod.version,mod.update,mod.patch)
			valid.append(ac)
except OSError: 
	print("no app manifest")

# == Method 1 == 
l=os.listdir('apps')

for file in l:
	if(file.endswith('.py') and (file[0] != '.' and file != '_')): #Remove duplicates
		newfile = file[:-3] #remove extension
		modName = "apps/" + newfile
		try:
			mod = __import__(modName)				#import from name        
		except Exception as err:        
			print("Error occurred importing Python module "+str(newfile)+": "+str(err))
		else:
			try:
				cor=mod.main #lack of () is important; we don't want to call constructor yet	
			except Exception as err:
				print("'"+modName+"' lacks a main function: "+str(err))
			else:
				ac = AppContainer(cor, False)
				ac.setAttributes(newfile.replace('_', ' '), '',1,0,0)
				valid.append(ac)

	
loop = asyncio.get_event_loop()
loop.set_exception_handler(_handle_exception_and_stop)

r=remote.Remote()
mt=None

try:
	f = open('apps/radio.conf', 'rb')
	(ch, uuid) = unpack('>II',f.read())
	print("ch="+hex(ch)+" uuid="+hex(uuid))
	f.close()
	r.getRadio().set_channel(ch)
	r.getRadio().set_uuid(uuid)
except OSError:
	print("no radio settings saved")


def drawWindow():
	pairstat = 'Not Paired'
	if( r.is_paired() ):
		pairstat = 'Paired: {:x}'.format(int(r.getRadio().get_channel()))
	
	#graphics.setViewport(0,0,graphics.lcdwidth,graphics.lcdheight)#OS context
	lpair.setText(pairstat)
	#graphics.light(100)					#full power
	graphics.fill_rectangle(0,0,graphics.lcdwidth,graphics.lcdheight,bgColor)
	graphics.fill_rectangle(0,0,graphics.lcdwidth,15,statusBgColor)	#create bar
	lsb.paint()
	lpair.paint()
	lbatt.paint()	
	graphics.rectangle(8,border_offset_y,graphics.lcdwidth-16,graphics.lcdheight-8-border_offset_y, textColor)
	y=30
	graphics.print("Select an item from the list", 15,y, textColor)
	mainmenu.paint()

def fStopApp():
	global mt
	global appRunning
	print("stop app!")
	if(not mt.cancel()):
		print("failed to cancel!!")
	appRunning=False
	selectCallback.set(mainmenu.doSelect)
	upCallback.set(mainmenu.doUp)
	downCallback.set(mainmenu.doDown)
	backCallback.set(mainmenu.doBack)
	graphics.setViewport(0,0,graphics.lcdwidth,graphics.lcdheight)
	drawWindow()

def runApp(app_container):
	global mt
	global appRunning
	if(app_container is not None):
		print("runApp: "+str(app_container))
		selectCallback.set(None)
		upCallback.set(None)
		downCallback.set(None)
		backCallback.set(fStopApp)
		graphics.setViewport(0,15,graphics.lcdwidth,graphics.lcdheight-15)
		graphics.fill_rectangle(0,0,graphics.lcdwidth,graphics.lcdheight,bgColor)
		graphics.print("Running "+app_container.name, 10,100, textColor)
		coro =None
		#construct a coro (passing no args) based on relevant method
		if(app_container.isClass):
			coro = app_container.app().main()					
		else:
			coro = app_container.app()
		mt=loop.create_task(coro)					#run main in a loop
		print("created task: "+str(mt.coro))
		appRunning=True
	return None

toolsMenuItems=[]

def fPair(_):
	graphics.fill_rectangle(0,0,graphics.lcdwidth,graphics.lcdheight,bgColor)
	graphics.print("Press remote pairing button ...", 20,100, textColor)
	if r.pair():
		ch=r.getRadio().get_channel()
		uuid=r.getRadio().get_uuid()
		print("ch="+hex(ch)+" uuid="+hex(uuid))
		f = open('apps/radio.conf', 'wb')
		f.write(pack('>II',ch,uuid))
		f.close()
		graphics.print("Success!", 75,110, textColor)
	else:
		graphics.print("Timeout!", 75,110, textColor)
	time.sleep_ms(3000)
	drawWindow()
	return None

def fTestR(_):
	global mt
	global appRunning
	selectCallback.set(None)
	upCallback.set(None)
	downCallback.set(None)
	backCallback.set(fStopApp)
	#graphics.fill_rectangle(0,0,graphics.lcdwidth,15,statusBgColor)	#create bar
	#lsb.paint()
	#lpair.paint()
	#lbatt.paint()
	graphics.setViewport(0,15,graphics.lcdwidth,graphics.lcdheight-15)
	graphics.fill_rectangle(0,0,graphics.lcdwidth,graphics.lcdheight,bgColor)
	
	coro = remotetest.main()					
	mt=loop.create_task(coro)					#run main in a loop
	appRunning=True
	
	return None

def fRC(_):
	global mt
	global appRunning
	upCallback.set(None)
	downCallback.set(None)
	backCallback.set(fStopApp)
	graphics.setViewport(0,15,graphics.lcdwidth,graphics.lcdheight-15)
	graphics.fill_rectangle(0,0,graphics.lcdwidth,graphics.lcdheight,bgColor)

	coro = RC.main()					
	mt=loop.create_task(coro)					#run main in a loop
	appRunning=True
	
	return None

def fGlobalCommander(_):
	global mt
	global appRunning
	upCallback.set(None)
	downCallback.set(None)
	backCallback.set(fStopApp)
	graphics.setViewport(0,15,graphics.lcdwidth,graphics.lcdheight-15)
	graphics.fill_rectangle(0,0,graphics.lcdwidth,graphics.lcdheight,bgColor)

	coro = globalCommander.main()					
	mt=loop.create_task(coro)					#run main in a loop
	appRunning=True
	
	return None


def fExit(_):
	global exit
	exit=True
	graphics.fill_rectangle(0,0,graphics.lcdwidth,graphics.lcdheight,bgColor)
	graphics.print("Exited to Python REPL", 70,100, textColor)
	return None

def fGoBack():
	selectCallback.set(mainmenu.doSelect)
	upCallback.set(mainmenu.doUp)
	downCallback.set(mainmenu.doDown)
	backCallback.set(mainmenu.doBack)
	drawWindow()
	return None

def fSerial(_):
	selectCallback.set(None)
	upCallback.set(None)
	downCallback.set(None)
	backCallback.set(fGoBack)
	graphics.fill_rectangle(0,15,graphics.lcdwidth,graphics.lcdheight-15,bgColor)
	b = machine.unique_id()
	id=(b[11],b[10]+b[2],b[9],b[8]+b[0],b[7],b[6])
	graphics.print("SB2 S/N: "+''.join('{:02x}'.format(x) for x in id), 15,40, textColor)
	import sb2os
	graphics.print("SB2 OS: "+sb2os.version(), 15,60, textColor)
	rev=sb.getChargerFWRev()
	graphics.print("Charger FW Rev: "+str(rev>>8)+"."+str(rev&0xff), 15,80, textColor)
	t=os.uname()
	graphics.print("MicroPython Info", 15,120, textColor)
	graphics.print("-Release: "+t[2], 15,140, textColor)
	graphics.print("-Build id: "+t[3].split(' on ')[0], 15,160, textColor)
	graphics.print("-Build date: "+t[3].split(' on ')[1], 15,180, textColor)
	graphics.print("-Machine: "+t[4], 15,200, textColor)	
	return None

toolsMenuItems.append(MenuItem("Pair Remote", fPair))
toolsMenuItems.append(MenuItem("Test Remote", fTestR))
toolsMenuItems.append(MenuItem("Remote Control",fRC))
#toolsMenuItems.append(MenuItem("Global Commander (beta)",fGlobalCommander))
toolsMenuItems.append(MenuItem("System Info",fSerial))
toolsMenuItems.append(MenuItem("Exit GUI to REPL", fExit))

mainmenu=Menu(20,50,270,170)
len_valid = min(len(valid),8) #we arbitarily limit to 8 apps 
for i in range(len_valid):
	s=valid[i]
	print(str(s))
	mainmenu.addMenuItem(s.name+" "+s.author+" "+str(s.version)+"."+str(s.update)+"."+str(s.patch), runApp, s)

for i in range(8-len_valid):
	mainmenu.addMenuItem("<Empty>", lambda _: None)

mainmenu.addMenuItem("Tools", lambda _: toolsMenuItems)

selectCallback=KeyCallback(mainmenu.doSelect)
upCallback=KeyCallback(mainmenu.doUp)
downCallback=KeyCallback(mainmenu.doDown)
backCallback=KeyCallback(mainmenu.doBack)

async def mainloop():
	global exit
		
	drawWindow()
	
	while(not exit):
		await asyncio.sleep_ms(100)

#PC side tools can wait for this print statement after a reboot
print("Starting main loop")
loop.create_task(keys())
loop.create_task(status())
loop.run_until_complete(mainloop())

